@@ -1,5 +1,45 @@
 
 #include <tiny_tds_ext.h>
+#include <stdint.h>
+
+// TINY_TDS_MAX_TIME
+
+#if (SIZEOF_INT < SIZEOF_LONG) || defined(HAVE_RUBY_ENCODING_H)
+  /* On 64bit platforms we can handle dates way outside 2038-01-19T03:14:07 */
+  /* (10000*31557600) + (12*2592000) + (31*86400) + (11*3600) + (59*60) + 59 */
+  #define TINY_TDS_MAX_TIME 315607276799ULL
+#else
+  /* On 32bit platforms the maximum date the Time class can handle is 2038-01-19T03:14:07 */
+  /* 2038 years + 1 month + 19 days + 3 hours + 14 minutes + 7 seconds = 64318634047 seconds */
+  /* (2038*31557600) + (1*2592000) + (19*86400) + (3*3600) + (14*60) + 7 */
+  #define TINY_TDS_MAX_TIME 64318634047ULL
+#endif
+
+
+// TINY_TDS_MIN_TIME
+
+#if defined(HAVE_RUBY_ENCODING_H)
+  /* Ruby 1.9 */
+  /* 0000-1-1 00:00:00 UTC */
+  /* (0*31557600) + (1*2592000) + (1*86400) + (0*3600) + (0*60) + 0 */
+  #define TINY_TDS_MIN_TIME 2678400ULL
+#elif SIZEOF_INT < SIZEOF_LONG
+  /* 64bit Ruby 1.8 */
+  /* 0139-1-1 00:00:00 UTC */
+  /* (139*31557600) + (1*2592000) + (1*86400) + (0*3600) + (0*60) + 0 */
+  #define TINY_TDS_MIN_TIME 4389184800ULL
+#elif defined(NEGATIVE_TIME_T)
+  /* 1901-12-13 20:45:52 UTC : The oldest time in 32-bit signed time_t. */
+  /* (1901*31557600) + (12*2592000) + (13*86400) + (20*3600) + (45*60) + 52 */
+  #define TINY_TDS_MIN_TIME 60023299552ULL
+#else
+  /* 1970-01-01 00:00:01 UTC : The Unix epoch - the oldest time in portable time_t. */
+  /* (1970*31557600) + (1*2592000) + (1*86400) + (0*3600) + (0*60) + 1 */
+  #define TINY_TDS_MIN_TIME 62171150401ULL
+#endif
+
+
+// File Types/Vars
 
 VALUE cTinyTdsResult;
 extern VALUE mTinyTds, cTinyTdsClient, cTinyTdsError;
@@ -8,7 +48,7 @@ VALUE opt_decimal_zero, opt_float_zero, opt_one, opt_zero, opt_four, opt_19hdr,
 int   opt_ruby_186;
 static ID intern_new, intern_utc, intern_local, intern_localtime, intern_merge, 
           intern_civil, intern_new_offset, intern_plus, intern_divide, intern_Rational;
-static ID sym_symbolize_keys, sym_as, sym_array, sym_cache_rows, sym_first, sym_timezone, sym_local, sym_utc;
+static ID sym_symbolize_keys, sym_as, sym_array, sym_cache_rows, sym_first, sym_timezone, sym_local, sym_utc, sym_empty_sets;
 
 
 // Lib Macros
@@ -200,8 +240,9 @@ static VALUE rb_tinytds_result_fetch_row(VALUE self, ID timezone, int symbolize_
               msec  = date_rec.datemsecond;
           if (year+month+day+hour+min+sec+msec != 0) {
             VALUE offset = (timezone == intern_local) ? rwrap->local_offset : opt_zero;
+            uint64_t seconds = (year*31557600ULL) + (month*2592000ULL) + (day*86400ULL) + (hour*3600ULL) + (min*60ULL) + sec;
             /* Use DateTime */
-            if (year < 1902 || year+month+day > 2058) {
+            if (seconds < TINY_TDS_MIN_TIME || seconds > TINY_TDS_MAX_TIME) {
               VALUE datetime_sec = INT2NUM(sec);
               if (msec != 0) {
                 if ((opt_ruby_186 == 1 && sec < 59) || (opt_ruby_186 != 1)) {
@@ -256,8 +297,12 @@ static VALUE rb_tinytds_result_fields(VALUE self) {
   VALUE fields_processed = rb_ary_entry(rwrap->fields_processed, rwrap->number_of_results);
   if ((dbsqlok_rc == SUCCEED) && (dbresults_rc == SUCCEED) && (fields_processed == Qnil)) {
     /* Default query options. */
+    int symbolize_keys = 0, empty_sets = 1;
     VALUE qopts = rb_iv_get(self, "@query_options");
-    int symbolize_keys = (rb_hash_aref(qopts, sym_symbolize_keys) == Qtrue) ? 1 : 0;
+    if (rb_hash_aref(qopts, sym_symbolize_keys) == Qtrue)
+       symbolize_keys = 1;
+    if (rb_hash_aref(qopts, sym_empty_sets) == Qfalse)
+      empty_sets = 0;
     /* Set number_of_fields count for this result set. */
     rwrap->number_of_fields = dbnumcols(rwrap->client);
     if (rwrap->number_of_fields > 0) {
@@ -292,7 +337,7 @@ static VALUE rb_tinytds_result_each(int argc, VALUE * argv, VALUE self) {
   /* Local Vars */
   VALUE qopts, opts, block;
   ID timezone;
-  int symbolize_keys = 0, as_array = 0, cache_rows = 0, first = 0;
+  int symbolize_keys = 0, as_array = 0, cache_rows = 0, first = 0, empty_sets = 0;
   /* Merge Options Hash To Query Options. Populate Opts & Block Var. */
   qopts = rb_iv_get(self, "@query_options");
   if (rb_scan_args(argc, argv, "01&", &opts, &block) == 1)
@@ -315,6 +360,8 @@ static VALUE rb_tinytds_result_each(int argc, VALUE * argv, VALUE self) {
     rb_warn(":timezone option must be :utc or :local - defaulting to :local");
     timezone = intern_local;
   }
+  if (rb_hash_aref(qopts, sym_empty_sets) == Qtrue)
+    empty_sets = 1;
   /* Make The Results Or Yield Existing */
   if (NIL_P(rwrap->results)) {
     rwrap->results = rb_ary_new();
@@ -322,8 +369,9 @@ static VALUE rb_tinytds_result_each(int argc, VALUE * argv, VALUE self) {
     RETCODE dbresults_rc = rb_tinytds_result_dbresults_retcode(self);
     while ((dbsqlok_rc == SUCCEED) && (dbresults_rc == SUCCEED)) {
       int has_rows = (DBROWS(rwrap->client) == SUCCEED) ? 1 : 0;
-      rb_tinytds_result_fields(self);
-      if (has_rows && rwrap->number_of_fields > 0) {
+      if (has_rows || empty_sets || (rwrap->number_of_results == 0))
+        rb_tinytds_result_fields(self);
+      if ((has_rows || empty_sets) && rwrap->number_of_fields > 0) {
         /* Create rows for this result set. */
         unsigned long rowi = 0;
         VALUE result = rb_ary_new();
@@ -356,6 +404,7 @@ static VALUE rb_tinytds_result_each(int argc, VALUE * argv, VALUE self) {
         // If we find results increment the counter that helpers use and setup the next loop.
         rwrap->number_of_results = rwrap->number_of_results + 1;
         dbresults_rc = rb_tinytds_result_dbresults_retcode(self);
+        rb_ary_store(rwrap->fields_processed, rwrap->number_of_results, Qnil);
       } else {
         // If we do not find results, side step the rb_tinytds_result_dbresults_retcode helper and 
         // manually populate its memoized array while nullifing any memoized fields too before loop.
@@ -473,6 +522,7 @@ void init_tinytds_result() {
   sym_local = ID2SYM(intern_local);
   sym_utc = ID2SYM(intern_utc);
   sym_timezone = ID2SYM(rb_intern("timezone"));
+  sym_empty_sets = ID2SYM(rb_intern("empty_sets"));
   /* Data Conversion Options */
   opt_decimal_zero = rb_str_new2("0.0");
   rb_global_variable(&opt_decimal_zero);
